package Plugins::Rhapsody;

use Data::Dumper;
use LWP::UserAgent;
use POSIX qw(strftime);
use XML::Simple;
use URI::Escape;
use SOAP::Lite maptype => {};
use HTML::Entities;

use strict;

###############################################################################
sub init {
    main::plugin_register("Rhapsody", "Rhapsody Support", "/Rhapsody", \&Plugins::Rhapsody::html);
    main::http_register_handler("/Rhapsody/index", \&Plugins::Rhapsody::index);

    main::sonos_add_hook("SEARCH", \&Plugins::Rhapsody::search);
    main::sonos_add_hook("CONTAINER_RDCPA", \&Plugins::Rhapsody::container);
    main::sonos_add_hook("ITEM_RDCPA", \&Plugins::Rhapsody::item);
    main::sonos_add_hook("ITEM_RDCPI", \&Plugins::Rhapsody::item);
    main::sonos_add_hook("META_RDCPA", \&Plugins::Rhapsody::meta);
    main::sonos_add_hook("META_RDCPI", \&Plugins::Rhapsody::meta);
    main::sonos_add_waiting("SERVICES", "*", \&Plugins::Rhapsody::services);

    $Plugins::Rhapsody::userAgent = LWP::UserAgent->new(timeout => 5);

    $Plugins::Rhapsody::cobrandId = SOAP::Data->name("cobrandId")->value(40134)->type("");
    $Plugins::Rhapsody::filterRightsKey = SOAP::Data->name("filterRightsKey")->value(2)->type("");
    $Plugins::Rhapsody::developerKey = SOAP::Data->name("developerKey")->value("9F7E8I1D3I6H2B0I")->type("");
    $Plugins::Rhapsody::priority = SOAP::Data->name("priority")->value("5")->type("");
    $Plugins::Rhapsody::createdTop = 0;
}

###############################################################################
sub services {
    main::sonos_add_waiting("SERVICES", "*", \&Plugins::Rhapsody::add_top_containers);
    add_top_containers();
}
###############################################################################
sub add_top_containers {
    return if ($Plugins::Rhapsody::createdTop);
    return if (!exists $main::SERVICES{Rhapsody});
    return if (!$main::PREFS{Rhapsody}->{benabled});

    $Plugins::Rhapsody::createdTop = 1;

    main::sonos_mkcontainer ("", "object.container", "Rhapsody Genres", "RDCPA:GLBGENRES");
    main::sonos_mkcontainer ("", "object.container", "Rhapsody Charts", "RDCPA:GNRCHARTS");
    main::sonos_mkcontainer ("", "object.container", "Rhapsody Stations", "RDCPA:GLBSTATIONS");

    main::sonos_mkcontainer ("RDCPA:GNRCHARTS", "object.container", "Top Artists", "RDCPA:GNRTOPARTIST");
    main::sonos_mkcontainer ("RDCPA:GNRCHARTS", "object.container", "Top Albums", "RDCPA:GNRTOPALBUM");
    main::sonos_mkcontainer ("RDCPA:GNRCHARTS", "object.container", "Top Tracks", "RDCPA:GNRTOPTRACK");
}

###############################################################################
sub del_top_containers {
    return if (!$Plugins::Rhapsody::createdTop);

    $Plugins::Rhapsody::createdTop = 0;


    main::sonos_mkcontainer ("", "object.container", "Rhapsody Genres", "RDCPA:GLBGENRES");
    main::sonos_mkcontainer ("", "object.container", "Rhapsody Charts", "RDCPA:GNRCHARTS");
    main::sonos_mkcontainer ("", "object.container", "Rhapsody Stations", "RDCPA:GLBSTATIONS");

    main::sonos_mkcontainer ("RDCPA:GNRCHARTS", "object.container", "Top Artists", "RDCPA:GNRTOPARTIST");
    main::sonos_mkcontainer ("RDCPA:GNRCHARTS", "object.container", "Top Albums", "RDCPA:GNRTOPALBUM");
    main::sonos_mkcontainer ("RDCPA:GNRCHARTS", "object.container", "Top Tracks", "RDCPA:GNRTOPTRACK");
}

###############################################################################
sub quit {
}

###############################################################################
sub container {
    my ($what, $name) = @_;

    my $i = 1;
    my $tree;


#Only cache items for 12 hours
    if ((exists $Plugins::Rhapsody::CONTAINERS{$name}) && 
        ((time() - $Plugins::Rhapsody::CONTAINERS{$name}) > 12*60*60)) {

        delete $main::CONTAINERS{$name};
    }


    return undef if (exists $main::CONTAINERS{$name});

    my $startnum = 0;
    my $endnum = 500;
    my $type = $name;
    my $id = "";

    if ((my $semicolon = rindex($type, ";")) != -1) {
        $startnum = int(substr($type, $semicolon+1));
        $endnum = $startnum+500;
        $type = substr($type, 0, $semicolon);
    }

    my $start = SOAP::Data->name("start")->value($startnum)->type("");
    my $end = SOAP::Data->name("end")->value($endnum)->type("");

    $Plugins::Rhapsody::CONTAINERS{$name} = time();
    if ((my $colon = rindex($type, ":")) != 5) {
        $id = substr($type, $colon+1);
        $type = substr($type, 0, $colon+1);
    }

    if ($name eq "RDCPA:GNRTOPARTIST") {
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getTopArtists($start, $end, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        foreach my $artist (@{$tree->{artists}}) {
            main::sonos_mkcontainer($name, "object.container.person.musicArtist", "$i. ". decode_entities($artist->{name}), "RDCPA:GLBARTIST:" . $artist->{artistId}, "x-rincon-cpcontainer:RDCPA:GLBARTIST:" . $artist->{artistId});
            $i++;
        }
    } elsif ($name eq "RDCPA:GNRTOPALBUM") {
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getTopAlbums($start, $end, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        foreach my $album (@{$tree->{albums}}) {
            main::sonos_mkcontainer($name, "object.container.album.musicAlbum", "$i. ". decode_entities($album->{displayName}), "RDCPA:GLBALBUM:" . $album->{albumId}, "x-rincon-cpcontainer:RDCPA:GLBALBUM:" . $album->{albumId});
            $i++;

        }

    } elsif ($name eq "RDCPA:GNRTOPTRACK") {
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getTopTracks($start, $end, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        foreach my $track (@{$tree->{tracks}}) {
            main::sonos_mkcontainer($name, "object.item.audioItem.musicTrack", "$i. ". decode_entities($track->{name}), "RDCPI:GLBTRACK:" . $track->{trackId}, "radea:" . $track->{trackId} . ".wma");
            $i++;

        }
    } elsif ($type eq "RDCPA:GNRTOPARTIST:") {
        my $genreId = SOAP::Data->name("genreId")->value($id)->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getTopArtistsForGenre($genreId, $start, $end, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        foreach my $artist (@{$tree->{artists}}) {
            main::sonos_mkcontainer($name, "object.container.person.musicArtist", "$i. ". decode_entities($artist->{name}), "RDCPA:GLBARTIST:" . $artist->{artistId}, "x-rincon-cpcontainer:RDCPA:GLBARTIST:" . $artist->{artistId});
            $i++;
        }
    } elsif ($type eq "RDCPA:GNRTOPALBUM:") {
        my $genreId = SOAP::Data->name("genreId")->value($id)->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getTopAlbumsForGenre($genreId, $start, $end, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        foreach my $album (@{$tree->{albums}}) {
            main::sonos_mkcontainer($name, "object.container.album.musicAlbum", "$i. ". decode_entities($album->{displayName}), "RDCPA:GLBALBUM:" . $album->{albumId}, "x-rincon-cpcontainer:RDCPA:GLBALBUM:" . $album->{albumId});
            $i++;

        }

    } elsif ($type eq "RDCPA:GNRTOPTRACK:") {
        my $genreId = SOAP::Data->name("genreId")->value($id)->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getTopTracksForGenre($genreId, $start, $end, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        foreach my $track (@{$tree->{tracks}}) {
            main::sonos_mkcontainer($name, "object.item.audioItem.musicTrack", "$i. ". decode_entities($track->{name}), "RDCPI:GLBTRACK:" . $track->{trackId}, "radea:" . $track->{trackId} . ".wma");
            $i++;

        }
    } elsif ($type eq "RDCPA:ARTSAMPLER:") {
        my $artistId = SOAP::Data->name("artistId")->value($id)->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getTracksForArtistSampler($artistId, $start, $end, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        foreach my $track (@{$tree->{tracks}}) {
            main::sonos_mkcontainer($name, "object.item.audioItem.musicTrack", decode_entities($track->{name}), "RDCPI:GLBTRACK:" . $track->{trackId}, "radea:" . $track->{trackId} . ".wma");
        }
          
    } elsif ($type eq "RDCPA:ARTTOPTRACKS:") {
        my $artistId = SOAP::Data->name("artistId")->value($id)->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getTopTracksForArtist($artistId, $start, $end, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        foreach my $track (@{$tree->{tracks}}) {
            main::sonos_mkcontainer($name, "object.item.audioItem.musicTrack", "$i. ". decode_entities($track->{name}), "RDCPI:GLBTRACK:" . $track->{trackId}, "radea:" . $track->{trackId} . ".wma");
            $i++;
        }
          
    } elsif ($type eq "RDCPA:GLBARTIST:") {
        my $artistId = SOAP::Data->name("artistId")->value($id)->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getArtist($artistId, $Plugins::Rhapsody::developerKey, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        main::sonos_mkcontainer($name, "object.container.album.musicAlbum", "Top Tracks", "RDCPA:ARTSAMPLER:" . $tree->{artistId}, "x-rincon-cpcontainer:RDCPA:ARTSAMPLER:" . $tree->{artistId});

        main::sonos_mkcontainer($name, "object.container.album.musicAlbum", "Artist Sampler", "RDCPA:ARTTOPTRACKS:" . $tree->{artistId}, "x-rincon-cpcontainer:RDCPA:ARTTOPTRACKS:" . $tree->{artistId});

        foreach my $album (@{$tree->{albums}}) {
            main::sonos_mkcontainer($name, "object.container.album.musicAlbum", decode_entities($album->{displayName}), "RDCPA:GLBALBUM:" . $album->{albumId}, "x-rincon-cpcontainer:RDCPA:GLBALBUM:" . $album->{albumId});

        }
    } elsif ($type eq "RDCPA:GLBALBUM:") {
        my $albumId = SOAP::Data->name("albumId")->value($id)->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getAlbum($albumId, $Plugins::Rhapsody::developerKey, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        foreach my $track (@{$tree->{trackMetadatas}}) {
            main::sonos_mkcontainer($name, "object.item.audioItem.musicTrack", decode_entities($track->{name}), "RDCPI:GLBTRACK:" . $track->{trackId}, "radea:" . $track->{trackId} . ".wma");
        }
    } elsif ($name eq "RDCPA:GLBGENRES") {
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getRootGenre($Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        foreach my $genre (@{$tree->{childGenres}}) {
            main::sonos_mkcontainer($name, "object.container.genre.musicGenre", decode_entities($genre->{name}), "RDCPA:GLBSUBGENRE:" . $genre->{genreId});
        }
    } elsif ($type eq "RDCPA:GLBSUBGENRE:") {
        my $genreId = SOAP::Data->name("genreId")->value($id)->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getGenre($genreId, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        main::sonos_mkcontainer($name, "object.container", "All Artists", "RDCPA:SUBGNRALLARTISTS:$id");
        main::sonos_mkcontainer($name, "object.container", "Charts", "RDCPA:GNRCHARTS:$id");
        main::sonos_mkcontainer($name, "object.container", "Key Artists", "RDCPA:SUBGNRKEYARTISTS:$id");
        main::sonos_mkcontainer($name, "object.container", "Key Albums", "RDCPA:SUBGNRKEYALBUMS:$id");
        main::sonos_mkcontainer($name, "object.container", "Genre Sampler", "RDCPA:SUBGNRSAMPLER:$id");
        if ($tree->{subGenreCount} > 0) {
            main::sonos_mkcontainer($name, "object.container", "Subgenres", "RDCPA:GLBLEAFGENRE:$id");
            foreach my $genre (@{$tree->{childGenres}}) {
                main::sonos_mkcontainer ("RDCPA:GLBLEAFGENRE:$id", "object.container.genre.musicGenre", decode_entities($genre->{displayName}), "RDCPA:GLBSUBGENRE:" . $genre->{genreId});
            }
        }

        main::sonos_mkcontainer ("RDCPA:GNRCHARTS:$id", "object.container", "Top Artists", "RDCPA:GNRTOPARTIST:$id");
        main::sonos_mkcontainer ("RDCPA:GNRCHARTS:$id", "object.container", "Top Albums", "RDCPA:GNRTOPALBUM:$id");
        main::sonos_mkcontainer ("RDCPA:GNRCHARTS:$id", "object.container", "Top Tracks", "RDCPA:GNRTOPTRACK:$id");
    } elsif ($name eq "RDCPA:GLBSTATIONS") {
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getProgrammedStations($start, $end, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        foreach my $station (@{$tree->{stations}}) {

            main::sonos_mkcontainer($name, "object.item.audioItem.audioBroadcast", decode_entities($station->{name}), "RDCPI:GLBSTATION:" . $station->{stationId}, "rdradio:station:" . $station->{stationId});
        }
    } elsif ($type eq "RDCPA:SUBGNRALLARTISTS:") {
        my $genreId = SOAP::Data->name("genreId")->value($id)->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getArtistsForGenreByPriority($genreId, $Plugins::Rhapsody::priority, $start, $end, $Plugins::Rhapsody::developerKey, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        if ($startnum > 0) {
            main::sonos_mkcontainer($name, "object.container", "--- Previous Page ---", "$type$id;" . ($startnum - 500));
        }

        foreach my $artist (@{$tree->{artists}}) {
            main::sonos_mkcontainer($name, "object.container.person.musicArtist", decode_entities($artist->{name}), "RDCPA:GLBARTIST:" . $artist->{artistId}, "x-rincon-cpcontainer:RDCPA:GLBARTIST:" . $artist->{artistId});
        }

        if ($endnum < $tree->{fullArraySize}) {
            main::sonos_mkcontainer($name, "object.container", "--- Next Page ---", "$type$id;$endnum");
        }
    } elsif ($type eq "RDCPA:SUBGNRKEYARTISTS:") {
        my $genreId = SOAP::Data->name("genreId")->value($id)->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getKeyArtistsForGenre($genreId, $start, $end, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        foreach my $artist (@{$tree->{artists}}) {
            main::sonos_mkcontainer($name, "object.container.person.musicArtist", decode_entities($artist->{name}), "RDCPA:GLBARTIST:" . $artist->{artistId}, "x-rincon-cpcontainer:RDCPA:GLBARTIST:" . $artist->{artistId});
        }
    } elsif ($type eq "RDCPA:SUBGNRKEYALBUMS:") {
        my $genreId = SOAP::Data->name("genreId")->value($id)->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getKeyAlbumsForGenre($genreId, $start, $end, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        foreach my $album (@{$tree->{albums}}) {
            main::sonos_mkcontainer($name, "object.container.album.musicAlbum", decode_entities($album->{displayName}), "RDCPA:GLBALBUM:" . $album->{albumId}, "x-rincon-cpcontainer:RDCPA:GLBALBUM:" . $album->{albumId});
        }
    } elsif ($type eq "RDCPA:SUBGNRSAMPLER:") {
        my $genreId = SOAP::Data->name("genreId")->value($id)->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getTracksForGenreSampler($genreId, $start, $end, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;
        foreach my $track (@{$tree->{tracks}}) {
            main::sonos_mkcontainer($name, "object.item.audioItem.musicTrack", decode_entities($track->{name}), "RDCPI:GLBTRACK:" . $track->{trackId}, "radea:" . $track->{trackId} . ".wma");
        }
    } else {
        main::Log (1, "Unknown container $name\n");
        return undef;
    }

    main::Log (4, Dumper($tree)) if (main::isLog(4));

    return undef;
}

###############################################################################
sub item {
    my ($what, $name) = @_;

    my $tree;
    if (substr($name, 0, 17) eq "RDCPA:ARTSAMPLER:") {
        main::sonos_mkitem ("RDCPA:GLBARTIST:" . substr($name,17), "object.container.album.musicAlbum", $name, "Artist Sampler", "x-rincon-cpcontainer:$name");
    } elsif (substr($name, 0, 19) eq "RDCPA:ARTTOPTRACKS:") {
        main::sonos_mkitem ("RDCPA:GLBARTIST:" . substr($name,19), "object.container.album.musicAlbum", $name, "Top Tracks", "x-rincon-cpcontainer:$name");
    } elsif (substr($name, 0, 16) eq "RDCPA:GLBARTIST:") {
        my $artistId = SOAP::Data->name("artistId")->value(substr($name,16))->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getArtist($artistId, $Plugins::Rhapsody::developerKey, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        main::sonos_mkitem ("", "object.container.person.musicArtist", $name, decode_entities($tree->{name}), "x-rincon-cpcontainer:RDCPA:GLBARTIST:" . $tree->{artistId});
    } elsif (substr($name, 0, 15) eq "RDCPA:GLBALBUM:") {
        my $albumId = SOAP::Data->name("albumId")->value(substr($name,15))->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getAlbum($albumId, $Plugins::Rhapsody::developerKey, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        main::sonos_mkitem ("", "object.container.album.musicAlbum", $name, decode_entities($tree->{displayName}), "x-rincon-cpcontainer:RDCPA:GLBALBUM:" . $tree->{albumId});
    } elsif (substr($name, 0, 15) eq "RDCPI:GLBTRACK:") {
        my $trackId = SOAP::Data->name("albumId")->value(substr($name,15))->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getTrack($trackId, $Plugins::Rhapsody::developerKey, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        main::sonos_mkitem ("RDCPA:GLBALBUM:" . $tree->{albumId}, "object.item.audioItem.musictrack", $name, decode_entities($tree->{name}), "radea:" . $tree->{trackId} . ".wma");

    } else {
        main::Log (1, "Unknown item $name\n");
        return undef;
    }

    main::Log (4, Dumper($tree)) if (main::isLog(4));

    return undef;
}
###############################################################################
sub meta {
    my ($what, $name, $entry) = @_;

    my $metadata = '&lt;DIDL-Lite xmlns:dc=&quot;http://purl.org/dc/elements/1.1/&quot; xmlns:upnp=&quot;urn:schemas-upnp-org:metadata-1-0/upnp/&quot; xmlns:r=&quot;urn:schemas-rinconnetworks-com:metadata-1-0/&quot; xmlns=&quot;urn:schemas-upnp-org:metadata-1-0/DIDL-Lite/&quot;&gt;&lt;item id=&quot;' . $entry->{id} . '&quot; parentID=&quot;' . $entry->{parentID} . '&quot; restricted=&quot;true&quot;&gt;&lt;dc:title&gt;'. $entry->{"dc:title"} .  '&lt;/dc:title&gt;&lt;upnp:class&gt;';

    if ($what eq "META_RDCPI") {
        $metadata .= 'object.item.audioItem.musicTrack';
    } else {
        $metadata .= 'object.container.album.musicAlbum';
    }
    $metadata .= '&lt;/upnp:class&gt;&lt;desc id=&quot;cdudn&quot; nameSpace=&quot;urn:schemas-rinconnetworks-com:metadata-1-0/&quot;&gt;' . $main::SERVICES{Rhapsody} . '&lt;/desc&gt;&lt;/item&gt;&lt;/DIDL-Lite&gt;';

    return decode_entities($metadata);
}
###############################################################################
sub search {
    my ($what, $zone, $results, $search, $maxsearch, $searchartist, $searchalbum, $searchsong) = @_;

    return if (!$main::PREFS{Rhapsody}->{senabled});

    my $url = "http://realsearch.real.com/search/?searchtype=RhapKeyword&query=" . $search . "&size=150&availability=Stream";
    my $response = $Plugins::Rhapsody::userAgent->get($url);

    if (!$response->is_success()) {
        main::Log(2, "Rhapsody search failed: ". Dumper($response));
        return undef;
    }
    my $content = $response->content;
    my $tree = XMLin($content, forcearray => ["artist", "album", "track"],
                               keyattr=>{"search-results"   => "searchcode"});

    main::Log (4, Dumper($tree)) if (main::isLog(4));

    if ($searchartist) {
        foreach my $artist (@{$tree->{"search-results"}->{artist}->{artists}->{artist}}) {
            last if (scalar @{$results} > $maxsearch);
            my %data;
            $data{MUSIC_ISSONG} = 0;
            $data{MUSIC_NAME} = encode_entities($artist->{"display-name"});
            $data{MUSIC_PATH} = uri_escape("RDCPA:GLBARTIST:Art." . $artist->{"artist-id"});
            if (defined $zone) {
                $data{MUSIC_ARG} = "zone=" . uri_escape($zone) . 
                                      "&amp;mpath=" . $data{MUSIC_PATH};
            }
            $data{MUSIC_ICON} = "/artist.gif";
            push(@{$results}, \%data);
        }
    }

    if ($searchalbum) {
        foreach my $album (@{$tree->{"search-results"}->{album}->{albums}->{album}}) {
            last if (scalar @{$results} > $maxsearch);
            my %data;
            $data{MUSIC_ISSONG} = 0;
            $data{MUSIC_NAME} = encode_entities($album->{"display-name"} . " - " . $album->{"artist-name"});
            $data{MUSIC_PATH} = uri_escape("RDCPA:GLBALBUM:Alb." . $album->{"album-content-id"});
            if (defined $zone) {
                $data{MUSIC_ARG} = "zone=" . uri_escape($zone) . 
                                      "&amp;mpath=" . $data{MUSIC_PATH};
            }
            $data{MUSIC_ICON} = "/album.gif";
            push(@{$results}, \%data);
        }
    }

    if ($searchsong) {
        foreach my $track (@{$tree->{"search-results"}->{track}->{tracks}->{track}}) {
            last if (scalar @{$results} > $maxsearch);
            my %data;
            $data{MUSIC_ISSONG} = 1;
            $data{MUSIC_NAME} = encode_entities($track->{"display-name"} . " - " . $track->{"album-title"} . " - " . $track->{"artist-name"});
            $data{MUSIC_PATH} = uri_escape("RDCPI:GLBTRACK:Tra." . $track->{"content-id"});
            if (defined $zone) {
                $data{MUSIC_ARG} = "zone=" . uri_escape($zone) . 
                                      "&amp;mpath=" . $data{MUSIC_PATH};
            }
            $data{MUSIC_ICON} = "/song.gif";
            push(@{$results}, \%data);
        }
    }

    return undef;
}

###############################################################################
sub html {
    my ($c, $r, $diskpath, $template) = @_;

    $template->param("RHAPSODY_CONFIGED" => exists $main::SERVICES{Rhapsody});
    $template->param("RHAPSODY_SENABLED" => "checked") if ($main::PREFS{Rhapsody}->{senabled});
    $template->param("RHAPSODY_BENABLED" => "checked") if ($main::PREFS{Rhapsody}->{benabled});

    undef $Plugins::Rhapsody::Err;
}

###############################################################################
sub index {
    my ($c, $r) = @_;

    my %qf = $r->uri->query_form;

    $main::PREFS{Rhapsody}->{benabled} = defined $qf{benabled};
    $main::PREFS{Rhapsody}->{senabled} = defined $qf{senabled};
    main::sonos_prefsdb_save();

    add_top_containers() if ($main::PREFS{Rhapsody}->{benabled});
    $c->send_redirect(main::http_base_url($r) . "/Rhapsody/index.html", HTTP::Status::RC_MOVED_TEMPORARILY);
    $c->force_last_request;
}

1;
